#!/usr/bin/env python
"""
_FindDASToUpload_

Oracle version of DBS3Buffer.FindDASToUpload
"""

from WMComponent.DBS3Buffer.MySQL.FindDASToUpload import FindDASToUpload as MySQLFindDASToUpload

class FindDASToUpload(MySQLFindDASToUpload):
    pass
