#!/usr/bin/env python
"""
_UpdateDataset_

Oracle implementation of DBS3Buffer.UpdateDataset

Created on May 15, 2013

@author: dballest
"""

from WMComponent.DBS3Buffer.MySQL.UpdateDataset import UpdateDataset as MySQLUpdateDataset

class UpdateDataset(MySQLUpdateDataset):
    pass
