#!/usr/bin/env python
"""
_MarkDatasetSubscribed_

Oracle implementation of MarkDatasetSubscribed.
"""

from __future__ import division

from WMComponent.RucioInjector.Database.MySQL.MarkDatasetSubscribed import MarkDatasetSubscribed as MySQLBase


class MarkDatasetSubscribed(MySQLBase):
    pass
