#!/usr/bin/env python
"""
_GetWorkflowTask_

Oracle implementation of Jobs.GetWorkflowTask
"""

from WMCore.WMBS.MySQL.Jobs.GetWorkflowTask import GetWorkflowTask as MySQLGetWorkflowTask

class GetWorkflowTask(MySQLGetWorkflowTask):
    pass
