#!/usr/bin/env python
"""
_CheckStatusForCompletedWorkflows_

Oracle implementation of DBSBuffer.CheckStatusForCompletedWorkflows
"""
from __future__ import print_function, division

from WMComponent.DBS3Buffer.MySQL.CheckStatusForCompletedWorkflows \
 import CheckStatusForCompletedWorkflows as MySQLCheckStatusForCompletedWorkflows

class CheckStatusForCompletedWorkflows(MySQLCheckStatusForCompletedWorkflows):
    """
    _CountFiles_

    """
    pass