#!/usr/bin/env python
"""
_ListIncomplete_

Oracle implementation of Subscription.ListIncomplete
"""

from WMCore.WMBS.MySQL.Subscriptions.ListIncomplete import ListIncomplete as ListIncompleteMySQL

class ListIncomplete(ListIncompleteMySQL):
    pass
